
###################################################################
####### computes KM
###################################################################
myOwnKM = function(time, delta, returnToOriginalOrder = TRUE) {
    ###-----------------------------------------------
    ### time - time to event delta - event indicator
    ### returnToOriginalOrder = TRUE - the order of time values is the
    ### same as in original data it also returns the delta (event
    ### indicator) returnToOriginalOrder = FALSE - the time is unique
    ### and ordered and the resulting data frame does not contain delta
    ### (event indicator).
    uniqueAndOrderedTime = unique(time)[order(unique(time))]
    nEvents = tapply(delta, time, sum)
    nDrop = tapply(delta, time, length)
    atRisk = c(length(time), length(time) - cumsum(nDrop))[1:length(nDrop)]
    probForEachTime = (1 - nEvents/atRisk)
    dataKM = data.frame(time = uniqueAndOrderedTime, nEvents = nEvents, 
        atRisk = atRisk, KM = cumprod(probForEachTime))
    dataKM$CDF = 1 - dataKM$KM
    dataKM$CDF_M = c(0, dataKM$CDF[1:(nrow(dataKM) - 1)])
    if (returnToOriginalOrder) {
        rownames(dataKM) = uniqueAndOrderedTime
        ### let's order the output according to the original order of time
        dataKM = dataKM[as.character(time), ]
        dataKM$delta = delta
    }
    rownames(dataKM) = 1:nrow(dataKM)
    dataKM
}

###################################################################
######### Dabrowska estimator  ####################################
###################################################################
survDabrowska = function(X, Y, deltaX, deltaY){
  data = data.frame(X = X, Y = Y, deltaX = deltaX, deltaY = deltaY)

  if(any(!c("X", "Y", "deltaX", "deltaY") %in% names(data))) stop("data.frame data has to contain the following variables: X, Y, deltaX, deltaY")
  if(nrow(data)<2) stop("Not enough data")
  X = data[,"X"]
  Y = data[,"Y"]
  deltaX = data[,"deltaX"]
  deltaY = data[,"deltaY"]
  
  ### Compute marginal survival curves:
  ###########################################################
  Sx = myOwnKM(time = X, delta = data[,"deltaX"])
  Sy = myOwnKM(time = Y, delta = data[,"deltaY"])
  SxUnique = unique(Sx[order(Sx$time), c("time", "KM")])
  SyUnique = unique(Sy[order(Sy$time), c("time", "KM")])
  
  ### Compute bivariate and univariate hazards:
  ###########################################################
  nX = nY = length(X)
  uniqueTimeX = unique(unlist(X))
  uniqueTimeY = unique(unlist(Y))
  numXTimes = length(uniqueTimeX)
  numYTimes = length(uniqueTimeY)
  Lam = matrix(NA, nrow=numXTimes, ncol=numYTimes)
  LamX = matrix(NA, nrow=numXTimes, ncol=numYTimes)
  LamY = matrix(NA, nrow=numXTimes, ncol=numYTimes)
  XEventMatr = matrix(NA, nrow=numXTimes, ncol=nX)
  YEventMatr = matrix(NA, nrow=numYTimes, ncol=nY)
  XAtRiskMatr =  matrix(NA, nrow=numXTimes, ncol=nX)
  YAtRiskMatr =  matrix(NA, nrow=numYTimes, ncol=nY)
  for(i in 1:nX){
    XAtRiskMatr[,i] = (X[i] >= uniqueTimeX)
    YAtRiskMatr[,i] = (Y[i] >= uniqueTimeY)
    XEventMatr[,i] = (X[i] == uniqueTimeX)*deltaX[i]
    YEventMatr[,i] = (Y[i] == uniqueTimeY)*deltaY[i]
  }
  atRiskMatr = XAtRiskMatr %*% t(YAtRiskMatr)
  M11 = XEventMatr %*% t(YEventMatr)
  M10 = XEventMatr %*% t(YAtRiskMatr)
  M01 = XAtRiskMatr %*% t(YEventMatr)
  
  ######## double hazard
  ###########################################################
  Lam = M11/atRiskMatr
  
  ######## single hazards
  ###########################################################
  LamX = M10 /atRiskMatr
  LamY = M01 /atRiskMatr

  ######## Dabrowska estimator:
  ###########################################################
  bigL = (LamX*LamY - Lam)/((1 - LamX)*(1 - LamY))
  indicesX = data.frame(rowNames = uniqueTimeX, rowIndex = 1:length(uniqueTimeX))
  indicesY = data.frame(colNames = uniqueTimeY, colIndex = 1:length(uniqueTimeY))
  indicesX = indicesX[order(uniqueTimeX), ]
  indicesY = indicesY[order(uniqueTimeY), ]
  DabrowskaEstNew = matrix(NA, nrow=numXTimes+1, ncol=numYTimes+1)
  DabrowskaEstNew[1, 1] = 1
  DabrowskaEstNew[2:nrow(DabrowskaEstNew), 1] = SxUnique["KM"][,1]
  DabrowskaEstNew[1, 2:ncol(DabrowskaEstNew)] = SyUnique["KM"][,1]
  colnames(DabrowskaEstNew) = c("0", indicesY$colNames)
  rownames(DabrowskaEstNew) = c("0", indicesX$rowNames)
  DabrowskaCDF = 1 - DabrowskaEstNew
  subBigLNonMiss = 1 - bigL
  subBigLNonMiss[is.na(subBigLNonMiss)] = 1
  rowInd = indicesX$rowIndex
  colInd = indicesY$colIndex
  for (i in 1:numXTimes){
    for (j in 1:numYTimes){
      DabrowskaEstNew[i+1, j+1] = (DabrowskaEstNew[i, j+1] * DabrowskaEstNew[i+1, j] / DabrowskaEstNew[i, j]) * subBigLNonMiss[rowInd[i], colInd[j]]
    }
  }
  ### Some values will be NaN because of division by zero (because we DabrowskaEstNew[i, j]
  ### = 0 at some point), so we will assign NA's to 0 
  ###########################################################
  DabrowskaEstNew[is.na(DabrowskaEstNew)] = 0
  
  ### Compute CDF
  ###########################################################
  DabrowskaCDF = 1 + DabrowskaEstNew - matrix(DabrowskaEstNew[1, ], nrow = nrow(DabrowskaEstNew), ncol = ncol(DabrowskaEstNew), byrow = TRUE) - 
  matrix(DabrowskaEstNew[, 1], nrow = nrow(DabrowskaEstNew), ncol = ncol(DabrowskaEstNew))
  list(DabrowskaEst = DabrowskaEstNew, DabrowskaCDF = DabrowskaCDF)
}

###################################################################
######### Get probability mass function from survival surface
###################################################################
survPMF = function(bivarSurf){  
  if(bivarSurf[1,1] != 1)
    {stop("Element bivarSurf[1,1] should be equal to 1")}
  numericX = as.numeric(rownames(bivarSurf))
  if(any(is.na(numericX)) | any(numericX != numericX[order(numericX)]) | numericX[1] != 0)
    {stop("Please check the row names of 'bivarSurf': they should be ORDERED NUMERIC values with FIRST element equal to '0'")}
  numericY = as.numeric(colnames(bivarSurf))
  if(any(is.na(numericY)) | any(numericY != numericY[order(numericY)]) | numericY[1] != 0)
    {stop("Please check the column names of 'bivarSurf': they should be ORDERED NUMERIC values with FIRST element equal to '0'")}
  
  nX = nrow(bivarSurf) - 1
  nY = ncol(bivarSurf) - 1
  unitVecX = matrix(1, ncol = nX)
  unitVecY = matrix(1, ncol = nY)

  Sx = matrix(as.numeric(bivarSurf[(1:nX)+1,1]), nrow = nX) %*% unitVecY
  Sy = t(matrix(as.numeric(bivarSurf[1,(1:nY)+1]), nrow = nY) %*% unitVecX)
  SxM = matrix(as.numeric(bivarSurf[1:nX,1]), nrow = nX) %*% unitVecY
  SyM = t(matrix(as.numeric(bivarSurf[1,1:nY]), nrow = nY) %*% unitVecX)
  Sdx = SxM - Sx
  Sdy = SyM - Sy

  SxMyM = rbind(rep(NA, nY + 1), cbind(rep(NA, nX), bivarSurf[1:nX, 1:nY]))
  SxM_y = rbind(rep(NA, nY+1), bivarSurf[1:nX, ])
  Sx_yM = cbind(rep(NA, nX+1), bivarSurf[, 1:nY])
  Sdx_y = SxM_y - bivarSurf
  Sx_dy = Sx_yM - bivarSurf
  Sdx_yM = SxMyM - Sx_yM
  SxM_dy = SxMyM - SxM_y
  Sdxdy = SxM_dy - Sx_dy
  rangeX = 1+(1:nX)
  rangeY = 1+(1:nY)
  
  pmfWithMarginalPMFs = Sdxdy
  rownames(pmfWithMarginalPMFs) = rownames(bivarSurf)
  colnames(pmfWithMarginalPMFs) = colnames(bivarSurf)
  pmfWithMarginalPMFs[,1] = c(0, Sdx[,1])
  pmfWithMarginalPMFs[1,] = c(0, Sdy[1,])
  
  returnRes = list(Sdxdy = pmfWithMarginalPMFs, Sxy = bivarSurf[rangeX, rangeY], SxMyM = SxMyM[rangeX, rangeY],  Sx = Sx, Sy = Sy, Sdx = Sdx, Sdy = Sdy, SxM = SxM, SyM = SyM, SxM_y = SxM_y[rangeX, rangeY], Sx_yM = Sx_yM[rangeX, rangeY], Sdx_y = Sdx_y[rangeX, rangeY], Sx_dy = Sx_dy[rangeX, rangeY], Sdx_yM = Sdx_yM[rangeX, rangeY], SxM_dy = SxM_dy[rangeX, rangeY])

  returnRes
}

#################################################################
######### Conditional survival surface (final version)
#################################################################
survRestricted = function(bivarSurf, tauX = Inf, tauY = Inf){
  Xs = as.numeric(rownames(bivarSurf))
  Ys = as.numeric(colnames(bivarSurf))
  if (tauX < min(Xs)) stop("tauY is out of bounds.")
  if (tauY < min(Ys)) stop("tauY is out of bounds.")

  deltaDiff = min(abs(c(diff(Xs), diff(Ys))))/2
  if(is.null(tauX)){tauX = max(Xs) + deltaDiff}
  if(is.null(tauY)){tauY = max(Ys) + deltaDiff}

  dimX = nrow(bivarSurf)
  dimY = ncol(bivarSurf)
  
  Fxy = bivarSurf*0
  Fxy[1,1] = 0
  Fxy[1, 2:dimY] = 1 - bivarSurf[1, 2:dimY]
  Fxy[2:dimX, 1] = 1 - bivarSurf[2:dimX, 1]
  Fxy[2:dimX, 2:dimY] = 1 - matrix(bivarSurf[1, 2:dimY], nrow = dimX-1, ncol = dimY-1, byrow = TRUE) - matrix(bivarSurf[2:dimX, 1], nrow = dimX-1, ncol = dimY-1) + bivarSurf[2:dimX, 2:dimY]
  
  transition = Fxy[as.numeric(rownames(Fxy)) < tauX, ]
  transition = transition[, as.numeric(colnames(transition)) < tauY]
  if(is.null(dim(transition))){
    stop("Not enough data points or the restricted region is too small.")
  }
  if(all(abs(transition[2:nrow(transition), 2:ncol(transition)]) < 10^(-13))){
    stop("Not enough data points or the restricted region is too small.")
  }  
  FxyCond = transition

  FxyCond[2:nrow(FxyCond), 2:ncol(FxyCond)] = FxyCond[2:nrow(FxyCond), 2:ncol(FxyCond)]/transition[nrow(transition), ncol(transition)]
  
  if(transition[nrow(transition), ncol(transition)] <= 0){
    cat("The probability of being in the restricted region is less or equal to zero, so the conditional survival probability cannot be computed.")
    res = list(Sxy = NA, SxMyM = NA,  Sx = NA, Sy = NA, Sdx = NA, Sdy = NA, SxM = NA, SyM = NA, SxM_y = NA, Sx_yM = NA, Sdx_y = NA, Sx_dy = NA, Sdx_yM = NA, SxM_dy = NA, Sdxdy = NA)
    return(res)
  }
  
  ### Computing the joint probability mass:
  #################################################################
  FxyCond[, 1] = 0
  FxyCond[1, ] = 0
  massMatr = FxyCond
  for(i in 2:nrow(FxyCond)){
    for(j in 2:ncol(FxyCond)){
      massMatr[i, j] = FxyCond[i, j] - FxyCond[i-1, j] - FxyCond[i, j-1] + FxyCond[i-1, j-1]
    }
  }
  
  ### Computing the marginal probability mass:
  #################################################################
  reducedRows = massMatr[2:nrow(massMatr), ]
  if(is.null(dim(reducedRows))){reducedRows = matrix(reducedRows, nrow = nrow(massMatr)-1, ncol = ncol(massMatr))}
  massMatr[1, ] = apply(reducedRows, 2, sum)
  reducedCols = massMatr[, 2:ncol(massMatr)]
  if(is.null(dim(reducedCols))){reducedCols = matrix(reducedCols, nrow = nrow(massMatr), ncol = ncol(massMatr) - 1)}
  massMatr[, 1] = apply(reducedCols, 1, sum)
  massMatr[1, 1] = 0
  
  ### Finishing conditional CDF:
  #################################################################
  FxyCond[1, ] = cumsum(massMatr[1, ])
  FxyCond[, 1] = cumsum(massMatr[, 1])
  
  ### Computing conditional survival surface:
  #################################################################
  SurvCond = FxyCond*0
  SurvCond[1,1] = 1
  SurvCond[2:nrow(SurvCond), 1] = 1 - FxyCond[2:nrow(FxyCond), 1]
  SurvCond[1, 2:ncol(SurvCond)] = 1 - FxyCond[1, 2:ncol(FxyCond)]
  SurvCond[2:nrow(SurvCond), 2:ncol(SurvCond)] = 1 - matrix(FxyCond[1, 2:ncol(SurvCond)], nrow = nrow(SurvCond)-1, ncol = ncol(SurvCond)-1, byrow = TRUE)- matrix(FxyCond[2:nrow(SurvCond), 1], nrow = nrow(SurvCond)-1, ncol = ncol(SurvCond)-1) + FxyCond[2:nrow(SurvCond), 2:ncol(SurvCond)]

  ### Prepare the rest of the surfaces
  #################################################################
  newDimX = nrow(SurvCond); newDimY = ncol(SurvCond)
  baseMatr = matrix(0, nrow = newDimX - 1, ncol = newDimY - 1)
  rownames(baseMatr) = rownames(SurvCond)[2:newDimX]
  colnames(baseMatr) = colnames(SurvCond)[2:newDimY]
  unitVecX = matrix(1, ncol = newDimX - 1)
  unitVecY = matrix(1, ncol = newDimY - 1)
  Sxy = Sx = Sy = Sdx = Sdy = SxM = SyM = SxM_y = Sx_yM = Sdx_y = Sx_dy = Sdx_yM = SxM_dy = SxMyM = Fxy = baseMatr
  
  Sxy = SurvCond[2:nrow(SurvCond), 2:ncol(SurvCond)]
  Sx = SurvCond[2:nrow(SurvCond), 1] %*% unitVecY
  Sy = t(SurvCond[1, 2:ncol(SurvCond)] %*% unitVecX)

  SxMyM = SurvCond[1:(newDimX-1), 1:(newDimY-1)]

  Sdx = -diff(c(1, Sx[, 1])) %*% unitVecY
  Sdy = t(-diff(c(1, Sy[1, ])) %*% unitVecX)
  SxM = Sdx + Sx; SyM = Sdy + Sy
  
  SxM_y = rbind(Sy[1, ], Sxy[1:(newDimX-2), ])
  Sx_yM = cbind(Sx[, 1], Sxy[, 1:(newDimY-2)])
  Sdx_y = SxM_y - Sxy; Sx_dy = Sx_yM - Sxy 
  Sdx_yM = SxMyM - Sx_yM; SxM_dy = SxMyM - SxM_y
  
  Sdxdy = Sdx_yM - Sdx_y
  
  res = list(Sxy = SurvCond, SxMyM = SxMyM,  Sx = Sx, Sy = Sy, Sdx = Sdx, Sdy = Sdy, SxM = SxM, SyM = SyM, SxM_y = SxM_y, Sx_yM = Sx_yM, Sdx_y = Sdx_y, Sx_dy = Sx_dy, Sdx_yM = Sdx_yM, SxM_dy = SxM_dy, Sdxdy = Sdxdy)
  
  for(n_i in setdiff(names(res), "Sxy")){
    rownames(res[[n_i]]) = NULL
    colnames(res[[n_i]]) = NULL
  }
  
  res
}

###################################################################
######### Highest Rank and Restrcited Spearman Rho in the Restricted Region
################################################################### 
survSpearman = function(X = NULL, Y = NULL, deltaX = NULL, deltaY = NULL, data = NULL, tauX = Inf, tauY = Inf, bivarSurf = NULL) {
    ### Arguments' check
    # if(method == "Dabrowska"){
    if(is.null(bivarSurf)){
      if(is.null(data)){
        l1 = length(X); l2 = length(Y); l3 = length(deltaX); l4 = length(deltaY)
        if(l1 != l2 | l1 != l3 | l1 != l4) stop("Arguments 'X', 'Y', 'deltaX', and 'deltaY' are numeric vectors of equal length when 'data' and 'bivarSurf' are NULL.\n'")
        data = data.frame(X = X, Y = Y, deltaX = deltaX, deltaY = deltaY)
      }
      if(nrow(data) == 0) stop("When bivarSurf' is not supplied either 'data' or 'X', 'Y', 'deltaX', and 'deltaY' should be provided with non-zero number of records")
      if(any(is.na(data))) stop("Missing values are not allowed in 'X', 'Y', 'deltaX', and 'deltaY'.\n")
      uniqueDeltas = unique(unlist(data[, c("deltaX", "deltaY")]))
      if(!(setequal(uniqueDeltas, c(0, 1)) | all(uniqueDeltas == 1) | all(uniqueDeltas == 0))) stop("Arguments 'deltaX', and 'deltaY' are binary.\n")
      if(all(data$deltaX == 0))stop("There are no events for X, all values of 'deltaX' = 0.\n")
      if(all(data$deltaY == 0))stop("There are no events for Y, all values of 'deltaY' = 0.\n")
      bivarSurf = survDabrowska(data$X, data$Y, data$deltaX, data$deltaY)$DabrowskaEst
    }else{
      #if(method != "Custom"){stop("Argument 'method' can be either 'Dabrowska' or 'Custom'.\n")}
      #if(is.null(bivarSurf)){stop("When 'method' is 'Custom', the user should supply a bivariate survival surface, argument 'bivarSurf', in the format described in the help file.\n")}
      if(any(is.na(bivarSurf))) stop("NA values are not allowed in 'bivarSurf'.\n")
      colN = as.numeric(colnames(bivarSurf))
      rowN = as.numeric(rownames(bivarSurf))
      if(length(colN)==0 | any(colN[order(colN)] != colN)) stop("Column names of 'bivarSurf' should be ordered X values.")
      if(length(rowN)==0 | any(rowN[order(rowN)] != rowN)) stop("Row names of 'bivarSurf' should be ordered Y values.")
      if(colN[1]!=0) stop("The first element of column names of 'bivarSurf' should be equal to 0.")
      if(rowN[1]!=0) stop("The first element of row names of 'bivarSurf' should be equal to 0.")
      if(bivarSurf[1,1] != 1) stop("Element 'bivarSurf[1,1]' should be equal to 1.")
    }
  
    ### Find the last observed events and censoring events
    lastX = rownames(bivarSurf)[nrow(bivarSurf)]
    lastY = colnames(bivarSurf)[ncol(bivarSurf)]
    lastEventOnX = names(bivarSurf[, 1] == min(bivarSurf[, 1]))[bivarSurf[, 
        1] == min(bivarSurf[, 1])][1]
    lastEventOnY = names(bivarSurf[1, ] == min(bivarSurf[1, ]))[bivarSurf[1, 
        ] == min(bivarSurf[1, ])][1]
    
    ### Find the last observed events and censoring events in the
    ### restricted region
    if(sum(as.numeric(rownames(bivarSurf)) < tauX) < 2){stop("'tauX' is too small: it excludes all points X.")}
    if(sum(as.numeric(colnames(bivarSurf)) < tauY) < 2){stop("'tauY' is too small: it excludes all points Y.")}
    restrBivarSurf = bivarSurf
    restrBivarSurf = restrBivarSurf[as.numeric(rownames(restrBivarSurf)) < 
        tauX, ]
    restrBivarSurf = restrBivarSurf[, as.numeric(colnames(restrBivarSurf)) < 
        tauY]
    restrLastX = rownames(restrBivarSurf)[nrow(restrBivarSurf)]
    restrLastY = colnames(restrBivarSurf)[ncol(restrBivarSurf)]
    restrLastEventOnX = names(restrBivarSurf[, 1] == min(restrBivarSurf[, 
        1]))[restrBivarSurf[, 1] == min(restrBivarSurf[, 1])][1]
    restrLastEventOnY = names(restrBivarSurf[1, ] == min(restrBivarSurf[1, 
        ]))[restrBivarSurf[1, ] == min(restrBivarSurf[1, ])][1]
    
    actualTauX = paste(restrLastEventOnX, "+")
    actualTauY = paste(restrLastEventOnY, "+")
    
    ### compute restricted correlation
    res1 = HighestRankAndRestrictedSpearman(bivarSurf, tauX = tauX, 
        tauY = tauY)
    bivarSurfForHR = bivarSurf
    bivarSurfForHR[as.numeric(rownames(bivarSurfForHR)) >= tauX, ] = 0
    bivarSurfForHR[, as.numeric(colnames(bivarSurfForHR)) >= tauY] = 0
    
    ### compute highest rank correlation in the restricted region
    res2 = HighestRankAndRestrictedSpearman(bivarSurfForHR, tauX = Inf, 
        tauY = Inf)
    
    resCor = c(HighestRank = res2["HighestRank", ], Restricted = res1["Restricted", 
        ])
    resRegUser = c(tauX = as.character(tauX), tauY = as.character(tauY))
    resReg = c(tauX = actualTauX, tauY = actualTauY)
    #list(RestrictedRegionSetByUser = resRegUser, RestrictedRegionEffective = resReg, 
    #    Correlation = resCor)
    list("Restricted region set by user" = resRegUser, "Effective restricted region" = resReg, 
         Correlation = resCor)
}

###################################################################
####### Overall and Restrcited Spearman's Rho
###################################################################
HighestRankAndRestrictedSpearman = function(bivarSurf, tauX = Inf, 
    tauY = Inf) {
    ######## computes 1) Overall Spearmans correlation using highest rank
    ######## approach 2) Restricted Spearmans correlation 'bivarSurf' is a
    ######## matrix representing a bivariate surface fit with the following
    ######## format: 1) the 1st row is the marginal survival function for Y
    ######## 2) the 1nd column is the marginal survival function for X 3)
    ######## bivarSurf[1,1] equals to 1 4) row names of 'bivarSurf' are
    ######## ordered X values including 0 5) column names of 'bivarSurf' are
    ######## ordered Y values including 0 Note: the covariance (the
    ######## numerator of the correlation) of the highest rank estimator can
    ######## be also computed using formula: Numerator = \sum_i \sum_j
    ######## S(x_i,y_i)S_X(dx_i)S_Y(dy_i) - S(x_i,dy_i)S_X(dx_i)S_Y(y_i) -
    ######## S(dx_i,y_i)S_X(x_i)S_Y(dy_i) + S(dx_i,dy_i)S_X(x_i)S_Y(y_i) The
    ######## above is equivalent to the highest rank estimator Numerator =
    ######## \sum _{i^*}\sum _{j^*} \left\{1 - \widehat{S}^H_X(x_{i^*})
    ######## - \widehat{S}^H_X(x_{i^*}^-) \right\} \left\{1 -
    ######## \widehat{S}^H_Y(y_{j^*}) - \widehat{S}^H_Y(y_{j^*}^-)
    ######## \right\} \widehat{S}^H(dx_{i^*}, dy_{j^*})
    
    ######## By setting the survival proability to zero outside of the data
    ######## support we effectively assume a left-over probability mass
    ######## which is what highest rank estimator is doing This ensures that
    ######## the above two estimators are the same
    oldrownames = rownames(bivarSurf)
    oldcolnames = colnames(bivarSurf)
    bivarSurfnew = bivarSurf
    bivarSurfnew = cbind(bivarSurfnew, rep(0, nrow(bivarSurfnew)))
    bivarSurfnew = rbind(bivarSurfnew, rep(0, ncol(bivarSurfnew)))
    rownames(bivarSurfnew) = c(oldrownames, "Inf")
    colnames(bivarSurfnew) = c(oldcolnames, "Inf")
    bivarSurf = bivarSurfnew
    ######## End of the part that ensures that the above two estimators are
    ######## the same
    
    Xs = as.numeric(rownames(bivarSurf))
    Ys = as.numeric(colnames(bivarSurf))
    rP1 = rP2 = c(NA, NA)
    
    ######## Identify the resticted region for X:
    maxX = max(Xs[Xs < tauX])
    maxY = max(Ys[Ys < tauY])
    if ((tauX <= Inf) & (sum(Xs < tauX) <= nrow(bivarSurf))) {
        rowsInRegion = Xs <= maxX
        rX1 = 1:(sum(rowsInRegion) - 1)
    } else {
        rX1 = 1:(nrow(bivarSurf) - 2)
    }
    rP1[1] = rX1[length(rX1)] + 1
    rP2[1] = rP1[1] + 1
    rX2 = rX1 + 1
    
    ######## Identify the resticted region for Y:
    if ((tauY <= Inf) & (sum(Ys < tauY) <= ncol(bivarSurf))) {
        colsInRegion = Ys <= maxY
        rY1 = 1:(sum(colsInRegion) - 1)
    } else {
        rY1 = 1:(ncol(bivarSurf) - 2)
    }
    rP1[2] = rY1[length(rY1)] + 1
    rP2[2] = rP1[2] + 1
    rY2 = rY1 + 1
    nX = nrow(bivarSurf) - 1
    nY = ncol(bivarSurf) - 1
    rangeX = 1 + (1:nX)
    rangeY = 1 + (1:nY)
    
    unitVecX = matrix(1, ncol = nX)
    unitVecY = matrix(1, ncol = nY)
    Sx = matrix(as.numeric(bivarSurf[(1:nX) + 1, 1]), nrow = nX) %*% 
        unitVecY
    Sy = t(matrix(as.numeric(bivarSurf[1, (1:nY) + 1]), nrow = nY) %*% 
        unitVecX)
    SxM = matrix(as.numeric(bivarSurf[1:nX, 1]), nrow = nX) %*% unitVecY
    SyM = t(matrix(as.numeric(bivarSurf[1, 1:nY]), nrow = nY) %*% 
        unitVecX)
    Sdx = SxM - Sx
    Sdy = SyM - Sy
    SxMyM = rbind(rep(NA, nY + 1), cbind(rep(NA, nX), bivarSurf[1:nX, 
        1:nY]))
    SxM_y = rbind(rep(NA, nY + 1), bivarSurf[1:nX, ])
    Sx_yM = cbind(rep(NA, nX + 1), bivarSurf[, 1:nY])
    Sdx_y = SxM_y - bivarSurf
    Sx_dy = Sx_yM - bivarSurf
    Sdx_yM = SxMyM - Sx_yM
    SxM_dy = SxMyM - SxM_y
    Sdxdy = SxM_dy - Sx_dy
    
    ### my favorite estimator
    numerator = sum(SxMyM[rangeX, rangeY] * Sdx * Sdy - SxM_dy[rangeX, 
        rangeY] * Sdx * SyM - Sdx_yM[rangeX, rangeY] * SxM * Sdy + 
        Sdxdy[rangeX, rangeY] * SxM * SyM)
    
    ### Highest rank estimator, which is equivalent to my favorite
    ### estimator when compu
    numerator0 = sum((2 * SxM - Sdx - 1) * (2 * SyM - Sdy - 1) * 
        Sdxdy[rangeX, rangeY], na.rm = TRUE)
    
    Sx_Stuff_Tail = c(Sx[, 1], 0)
    SxM_Stuff_Tail = c(1, Sx[, 1])
    Sdx_Stuff_Tail = SxM_Stuff_Tail - Sx_Stuff_Tail
    Sy_Stuff_Tail = c(Sy[1, ], 0)
    SyM_Stuff_Tail = c(1, Sy[1, ])
    Sdy_Stuff_Tail = SyM_Stuff_Tail - Sy_Stuff_Tail
    PSR_X_like = (1 - Sx_Stuff_Tail - SxM_Stuff_Tail)
    PSR_Y_like = (1 - Sy_Stuff_Tail - SyM_Stuff_Tail)
    PSR_X_mean = sum(PSR_X_like * Sdx_Stuff_Tail)
    PSR_Y_mean = sum(PSR_Y_like * Sdy_Stuff_Tail)
    PSR_X_var = sum(Sdx_Stuff_Tail * (PSR_X_like - PSR_X_mean)^2)
    PSR_Y_var = sum(Sdy_Stuff_Tail * (PSR_Y_like - PSR_Y_mean)^2)
    rhoConst = 1/sqrt(PSR_X_var * PSR_Y_var)
    
    ####### Compute overall rho
    localRhoRhoConst = rhoConst * numerator
    prsLikeNoTails = rhoConst * numerator0
    
    ####### Restricted Estimator
    ProbOmegaRMatr = 1 - Sx[rX1, rY1] - Sy[rX1, rY1] + bivarSurf[rX2, 
        rY2]
    if (length(rX1) == 1 | length(rY1) == 1) {
        ProbOmegaRMatr = matrix(ProbOmegaRMatr, nrow = length(rX1), 
            ncol = length(rY1))
        ProbOmegaR = ProbOmegaRMatr[nrow(ProbOmegaRMatr), ncol(ProbOmegaRMatr)]
    } else {
        ProbOmegaR = ProbOmegaRMatr[nrow(ProbOmegaRMatr), ncol(ProbOmegaRMatr)]
    }
    SX_tauXM = Sx[rX1[length(rX1)], 1]
    SY_tauYM = Sy[1, rY1[length(rY1)]]
    
    ####### Compute S(x, tauY-), S(x-, tauY-), S(tauX-, y), S(tauX-, y)
    S_x_TauYM = matrix(bivarSurf[rX2, rY2[length(rY2)]], nrow = length(rX2), 
        ncol = length(rY2))
    S_xM_TauYM = matrix(SxM_y[rX2, rY2[length(rY2)]], nrow = length(rX2), 
        ncol = length(rY2))
    S_TauXM_y = matrix(bivarSurf[rX2[length(rX2)], rY2], nrow = length(rX2), 
        ncol = length(rY2), byrow = TRUE)
    S_TauXM_yM = matrix(Sx_yM[rX2[length(rX2)], rY2], nrow = length(rX2), 
        ncol = length(rY2), byrow = TRUE)
    g_X_No_OmegaR = (2 - Sx[rX1, rY1] - SxM[rX1, rY1] - 2 * SY_tauYM + 
        S_x_TauYM + S_xM_TauYM - ProbOmegaR)
    g_Y_No_OmegaR = (2 - Sy[rX1, rY1] - SyM[rX1, rY1] - 2 * SX_tauXM + 
        S_TauXM_y + S_TauXM_yM - ProbOmegaR)
    
    ####### Compute SX(dx|OmegaR), SY(dy|OmegaR) for covariance According
    ####### to our paper, it is: SX(dx) - S(dx, tauYM)
    SX_dx_No_OmR = (Sdx[rX1, 1] - (S_xM_TauYM[, 1] - S_x_TauYM[, 
        1]))
    SY_dy_No_OmR = (Sdy[1, rY1] - (S_TauXM_yM[1, ] - S_TauXM_y[1, 
        ]))
    SX_dx_OmR = SX_dx_No_OmR/ProbOmegaR
    SY_dy_OmR = SY_dy_No_OmR/ProbOmegaR
    
    ####### Compute SX(dx|OmegaR), SY(dy|OmegaR) for rho (the normalizing
    ####### const) take care of negative variance
    SX_dx_No_OmR_const = pmax(SX_dx_No_OmR, 0)
    SY_dy_No_OmR_const = pmax(SY_dy_No_OmR, 0)
    SX_dx_No_OmR_const = ProbOmegaR * SX_dx_No_OmR_const/sum(SX_dx_No_OmR_const)
    SY_dy_No_OmR_const = ProbOmegaR * SY_dy_No_OmR_const/sum(SY_dy_No_OmR_const)
    
    ####### Compute terms related to variance
    var_g_X_No_OmR = sum(((g_X_No_OmegaR[, 1] - sum(g_X_No_OmegaR[, 
        1] * SX_dx_No_OmR)/ProbOmegaR)^2) * SX_dx_No_OmR)
    var_g_Y_No_OmR = sum(((g_Y_No_OmegaR[1, ] - sum(g_Y_No_OmegaR[1, 
        ] * SY_dy_No_OmR)/ProbOmegaR)^2) * SY_dy_No_OmR)
    
    ####### If the probability of being in the region is negative -> the
    ####### restricted correlation is not defined
    if (ProbOmegaR <= 0) {
        warning("Restricted Spearman's correlation is not defined because the probability of being in the restricted region is less or equal to zero.")
        rhoX0Y0_restr_No_OmR = NA
    } else {
        if (var_g_X_No_OmR < 0 | var_g_Y_No_OmR < 0) {
            cat("Restricted Spearman's correlation was corrected.\n")
            warning("Restricted Spearman's correlation was corrected.")
        }
        if (var_g_X_No_OmR < 0) {
            var_g_X_No_OmR = sum(((g_X_No_OmegaR[, 1] - sum(g_X_No_OmegaR[, 
                1] * SX_dx_No_OmR_const)/ProbOmegaR)^2) * SX_dx_No_OmR_const)
        }
        if (var_g_Y_No_OmR < 0) {
            var_g_Y_No_OmR = sum(((g_Y_No_OmegaR[1, ] - sum(g_Y_No_OmegaR[1, 
                ] * SY_dy_No_OmR_const)/ProbOmegaR)^2) * SY_dy_No_OmR_const)
        }
        
        ####### Compute the constant for restricted correlation:
        inv_rhoCons_cond_form_No_OmR = sqrt(var_g_X_No_OmR * var_g_Y_No_OmR)
        ####### Compute restricted correlation:
        rhoX0Y0_restr_No_OmR = sum(g_X_No_OmegaR * g_Y_No_OmegaR * 
            Sdxdy[rX2, rY2], na.rm = TRUE)/inv_rhoCons_cond_form_No_OmR
    }
    # res = matrix(c(localRhoRhoConst, prsLikeNoTails,
    # rhoX0Y0_restr_No_OmR), ncol = 1) rownames(res) =
    # c('HighestRank', 'HighestRankNoTails', 'Restricted')
    res = matrix(c(prsLikeNoTails, rhoX0Y0_restr_No_OmR), ncol = 1)
    rownames(res) = c("HighestRank", "Restricted")
    colnames(res) = "Spearman's Correlation"
    
    ####### Make sure that all values are in the proper range
    for (n_i in rownames(res)) {
        res[n_i, ] = min(abs(res[n_i, ]), 1) * sign(res[n_i, ])
    }
    res

}

###################################################################
####### Creates gray scale relatively to the max and min values
###################################################################
grayScale = function(x, X0, X1, Y0, Y1, method, lowerThreshold = -Inf, YValueForLowerThreshold = NA){
  ### there are points (X0,Y0) and (X1, Y1) that we aim for
  if(X0 == X1){
    stop("X0 should not be equal to X1.")
  }else{    
    if(!(method %in% c(1, 2, 3))){
      stop("Argument 'method' can equal only to 1, 2, or 3.")
    }
    if(method == 1){
      a = (Y0 - Y1)/(X0 - X1)
      b = Y1 - a*X1
      res = a*x  + b
    }
    if(method == 2){
      a = (Y1 - Y0)/((X1 - X0)^2)
      b = -2*a*X0
      c = Y0 + a*X0^2
      res = a*x^2  + b*x + c
    }
    if(method == 3){
      a = -(Y1 - Y0)/((X1 - X0)^2)
      b = -2*a*X1
      c = Y0 - a*X0^2 + 2*a*X1*X0
      res = a*x^2  + b*x + c
    }
  }
  res[x < lowerThreshold] = YValueForLowerThreshold
  res
}

###################################################################
########## good old functions for filling up zeros in decimals
###################################################################
fillUpDecimals = function(numVec, howManyToFill=NULL, fill = "0"){
  ### howManyToFill - if NULL then it fill up to the number of maximum decimal points
  ###   for example, if numStr = c(0.33, 34, 4.1) then it fills up to two decimal points
  ###   and the result will be a character string wiht c("0.33", "34.00", "4.10")
  # numStr = as.character(as.numeric(numVec))
  numStr = as.character(numVec)
  noWholeNum = gsub("^[-+ ]*[0-9a-zA-Z]*$", "", numStr, perl=TRUE)
  decimChar = gsub("^[-+ ]*[0-9a-zA-Z]*\\.", "", noWholeNum)
  if(is.null(howManyToFill)){
    howManyToFill = max(nchar(decimChar[!is.na(decimChar)]))
  }
  if(howManyToFill != 0){
    numToFill = pmax(0, howManyToFill - nchar(decimChar))
    fills = sapply(numToFill, function(x) {if(is.na(x)) "" else paste(rep(fill, x), collapse="")} )
    dPoints = sapply(nchar(noWholeNum), function(x){if(!is.na(x) & x==0) "." else ""})
    res = paste(numStr, dPoints, fills, sep="")
    res[res == "NA"] = NA
  }else{
    res = numStr

  }

  res

}

###################################################################
########## good old functions for filling up the str
###################################################################
fillUpStr = function(strVec, howManyToFill=NULL, where="head", fill = " "){
  ### "strVec" to fill
  ### howManyToFill - if NULL then it fill up to the number of maximum characters
  ### "where" can be either "head" or "tail"
  if(!any(where %in% c("head", "tail"))) stop("Argument 'where' can be either 'head' or 'tail'\n")
  if(is.null(howManyToFill)){
    howManyToFill = diff(range(nchar(strVec), na.rm = TRUE))
  }
  # numToFill = max(cnchar(strVec), na.rm = TRUE) - nchar(strVec)
  numToFill = max(max(nchar(strVec), na.rm = TRUE), howManyToFill) - nchar(strVec)
  numToFill[is.na(numToFill)] = 0
  fills = sapply(numToFill, function(x)paste(rep(fill, x), collapse=""))
  if (where == "head"){
    res = paste(fills, strVec, sep="")
  }else{
    res = paste(strVec, fills, sep="")
  }
  res[is.na(strVec)] = NA
  res
}

###################################################################
######### function used as a 1-D histogram
###################################################################
oneDimHistBreakCells = function(Sdx, divPointsX){
  ### Takes a one dimensional function (as a vector)
  ### this vector has to have names as points
  ### Takes division points: divPointsX
  ### and sums up the pieces of the vector based on these division points.
  if(FALSE){
    Sdx = c(1, 2, 3, 1, 2, 5, 9); names(Sdx) = as.character(1:length(Sdx))
    divPointsX = c(0, 3.5, 5, 5.1, 5.2, 8, 9)
    oneDimHistBreakCells(Sdx, divPointsX)
  }
  # browser()
  arrangeIndicesForHist = function(X, divPointsX){
    indicesX = matrix(NA, ncol = 2, nrow = length(divPointsX) - 1)
    # indicesX[1, 1] = 1
    i = iterX = 1
    while(iterX <= nrow(indicesX) & i <= length(X)){
      anythingInside = divPointsX[iterX] < X[i] & X[i] <= divPointsX[iterX+1]
      if(anythingInside){
        indicesX[iterX, 1] = i
        while( divPointsX[iterX] < X[i] & X[i] <= divPointsX[iterX+1] & i <= length(X)){
          i = i + 1
        }
        indicesX[iterX, 2] = i - 1
      }
      iterX = iterX + 1
    }
    divPX = cbind(divPointsX[1:(length(divPointsX) - 1)], divPointsX[2:(length(divPointsX))])
    list(divisP = divPX, indices = indicesX)
  }
  
  resX = arrangeIndicesForHist(as.numeric(names(Sdx)), divPointsX)
  indicesX = resX$indices
  
  newVec = rep(NA, nrow(indicesX))
  for(i in 1:nrow(indicesX)){
    if(all(is.na(indicesX[i, ]))){
      newVec[i] = 0
    }else{
      newVec[i] = sum(Sdx[indicesX[i, 1]:indicesX[i, 2]])
    }
  }
  list(newVec = newVec, divPX = resX$divisP)
}

###################################################################
######### function used as a 2-D histogram
###################################################################
twoDimHistBreakCells = function(Sdxdy, divPointsX, divPointsY){
  ### Takes a two dimensional function (as a matrix)
  ### this matrix has to have col and row names as points
  ### rows are Y's and cols are X's
  ### Takes division points:
  ### divPointsX are for colnames
  ### divPointsY are for rownames
  ### and sums up the pieces of the matrix based on these division points.
  
  arrangeIndicesForHist = function(X, divPointsX){
    indicesX = matrix(NA, ncol = 2, nrow = length(divPointsX) - 1)
    indicesX[1, 1] = 1
    i = iterX = 1
    while(iterX <= nrow(indicesX) & i <= length(X)){
      anythingInside = divPointsX[iterX] < X[i] & X[i] <= divPointsX[iterX+1]
      if(anythingInside){
        indicesX[iterX, 1] = i
        while( divPointsX[iterX] < X[i] & X[i] <= divPointsX[iterX+1] & i <= length(X)){
          i = i + 1
        }
        indicesX[iterX, 2] = i - 1
      }
      iterX = iterX + 1
    }
    divPX = cbind(divPointsX[1:(length(divPointsX) - 1)], divPointsX[2:(length(divPointsX))])
    list(divisP = divPX, indices = indicesX)
  }
  
  resX = arrangeIndicesForHist(as.numeric(rownames(Sdxdy)), divPointsX)
  resY = arrangeIndicesForHist(as.numeric(colnames(Sdxdy)), divPointsY)
  indicesX = resX$indices
  indicesY = resY$indices
  
  newMatr = matrix(NA, nrow = nrow(indicesX), ncol = nrow(indicesY))
  for(i in 1:nrow(indicesX)){
    for(j in 1:nrow(indicesY)){
      if(any(is.na(indicesX[i, ])) | any(is.na(indicesY[j, ]))){
        newMatr[i, j] = 0
      }else{
        newMatr[i, j] = sum(Sdxdy[indicesX[i, 1]:indicesX[i, 2], indicesY[j, 1]:indicesY[j, 2]])
      }
    }
  }
  list(newMatr = newMatr, divPX = resX$divisP, divPY = resY$divisP)
}

###################################################################
######### function that plots a 1-D histogram
###################################################################
plotVector = function(vectValues, width, coordX, coordY, rotationRadians = 0, vectColor = NULL, bordColor = NULL){
  if (length(width) == 1){
    width = rep(width, length(vectValues))
  }else{
    if (length(width) < length(vectValues) | length(width) <= 0) stop("The length of 'width' should be either 1 or the same as the length of 'vectValues'")
  }
  
  if(is.null(vectColor)){
    vectColor = gray(rep(0.2, length(vectValues)))
  }
  if (length(vectColor) == 1){
    vectColor = rep(vectColor, length(vectValues))
  }else{
    if (length(vectColor) < length(vectValues) | length(vectColor) <= 0) stop("The length of 'vectColor' should be either 1 or the same as the length of 'vectValues'")
  }
  
  if(is.null(bordColor)){
    bordColor = vectColor
  }
  if (length(bordColor) == 1){
    bordColor = rep(bordColor, length(vectValues))
  }else{
    if (length(bordColor) < length(vectValues) | length(bordColor) <= 0) stop("The length of 'bordColor' should be either 1 or the same as the length of 'vectValues'")
  }
  
  divPX = cbind(c(0, cumsum(width)[1:(length(width) - 1)]), cumsum(width))
  rotaionMatrix = matrix(c(cos(rotationRadians), - sin(rotationRadians), sin(rotationRadians), cos(rotationRadians)), nrow = 2, ncol = 2, , byrow = TRUE)
  
  ### checking the range of the plotted vector:
  xyRange = rotaionMatrix %*% rbind(c(coordX, coordX + sum(width)), c(coordY, coordY + max(vectValues)))
  usr <- par('usr')
  xr <- (usr[2] - usr[1]) / 27 # 27 = (100 + 2*4) / 4
  yr <- (usr[4] - usr[3]) / 27
  xlim <- c(usr[1] + xr, usr[2] - xr)
  ylim <- c(usr[3] + yr, usr[4] - yr)
  
  if((all(xyRange[,1] <= xlim[1])  |  all(xyRange[,1] >= xlim[2]))  &  (all(xyRange[,2] <= ylim[1])  |  all(xyRange[,2] >= ylim[2]))){
    warning("The plotted area in function 'plotVector':  [", paste(xyRange[,1], collapse = ", "), "]x[",  paste(xyRange[,2], collapse = ", "), "] is out of the following plotted range: [", paste(xlim, collapse = ", "), "]x[",  paste(ylim, collapse = ", "), "], and might not be displayed in the plot.")
  }
  
  for(i in 1:length(vectValues)){
    x = c(divPX[i, 1], divPX[i, 1], divPX[i, 2], divPX[i, 2])
    y = c(0, vectValues[i], vectValues[i], 0)
    newCoord = rotaionMatrix %*% rbind(x, y)
    polygon(newCoord[1,] + coordX, newCoord[2,] + coordY, border = bordColor[i], col = vectColor[i])
  
  }

}

###################################################################
######### function that plots a PDF of bivariate survival data
###################################################################
plotMatrix = function(colorMatr, borderCol = "white", coordX, coordY, widthX, widthY){
  ####### Plots a matrix as a set of rectangles with colors defined by the colorMatr
  ####### The columns ???
  ### colorMatr - matrix with color
  ### coordX, coordY - where to place the bottom left corder of the matrix
  ### widthX - the width of the plotted rectangles on X axis
  ### widthY - the width of the plotted rectangles on Y axis
  ### borderCol - the color of the rectanges border
  if(FALSE){
  if(class(colorMatr) != "matrix") stop("'colorMatr' is a matrix of character strings representing colors")
  if(lwX != nrow(colorMatr)) {stop("The number of rows in 'colorMatr' should be exactly the length of 'widthX'")}
  if(lwY != ncol(colorMatr)) {stop("The number of columns in 'colorMatr' should be exactly the length of 'widthY'")}
  
  xValues = coordX + c(0, cumsum(widthX))
  yValues = coordY + c(0, cumsum(widthY))
  for(i in 1:nrow(colorMatr)){
    for(j in 1:ncol(colorMatr)){
      polygon(x = c(xValues[i], xValues[i], xValues[i+1], xValues[i+1]), y = c(yValues[c(j, j+1)], yValues[c(j+1, j)]), border = borderCol, col = colorMatr[i,j])
    }
  }
  }
  
  if(class(colorMatr) != "matrix") stop("'colorMatr' is a matrix of character strings representing colors")
  lwX = length(widthX)
  lwY = length(widthY)
  if(lwX == 1){
    widthX = rep(widthX, ncol(colorMatr))
    lwX = length(widthX)
  }
  if(lwY == 1){
    widthY = rep(widthY, nrow(colorMatr))
    lwY = length(widthY)
  }
  if(lwX != ncol(colorMatr)) {stop("The length of 'widthX' should be 1 or the number of columns in 'colorMatr'")}
  if(lwY != nrow(colorMatr)) {stop("The length of 'widthY' should be 1 or the number of rows in 'colorMatr'")}

  xValues = coordX + c(0, cumsum(widthX))
  yValues = coordY + c(0, cumsum(widthY))
  yValues = yValues[length(yValues):1]
  for(i in 1:ncol(colorMatr)){
    for(j in 1:nrow(colorMatr)){
      polygon(x = c(xValues[i], xValues[i], xValues[i+1], xValues[i+1]), y = c(yValues[c(j, j+1)], yValues[c(j+1, j)]), border = borderCol, col = colorMatr[j,i])
    }
  }
  
}

###################################################################
######### function that plots a PDF of bivariate survival data
###################################################################
survPMFPlot = function(bivarSurf, gridWidthX = 0.025, gridWidthY = 0.025, scaleGapX = 1, scaleGapY = 1, XAxisLabel = "X", YAxisLabel = "Y", timeLabelScale = 1, axisLabelScale = 1, labelSkipX = 0, labelSkipY = 0, roundX = 2, roundY = 2, plotLabel = "Bivariate PMF"){
  
  whitestCol = 0.99; grayMaxCol = 0.00
  #maxX = max(sdata$X)
  #maxY = max(sdata$Y)
  maxX = as.numeric(rownames(bivarSurf))
  maxY = as.numeric(colnames(bivarSurf))
  totalMax = max(maxX, maxY)
  
  KMX = unlist(bivarSurf[2:nrow(bivarSurf), 1])
  KMY = unlist(bivarSurf[1, 2:ncol(bivarSurf)])
  tmpSurf = survPMF(bivarSurf)$Sdxdy
  Sdxdy = tmpSurf[2:nrow(tmpSurf), 2:ncol(tmpSurf)]
  Sdx = tmpSurf[2:nrow(tmpSurf), 1]
  Sdy = tmpSurf[1, 2:ncol(tmpSurf)]
  Sdx[Sdx < 0] = 0 
  Sdy[Sdy < 0] = 0 
  #################################
  ### get rid of negative mass:
  
  divPointsX = seq(0, max(gridWidthX*ceiling(maxX/gridWidthX), maxX), gridWidthX)
  divPointsX = divPointsX[order(divPointsX)]
  divPointsY = seq(0, max(gridWidthY*ceiling(maxY/gridWidthY), maxY), gridWidthY)
  divPointsY = divPointsY[order(divPointsY)]
  timeRangeX = divPointsX[seq(1, length(divPointsX), (labelSkipX+1))]
  timeRangeY = divPointsY[seq(1, length(divPointsY), (labelSkipY+1))]
  timeRangeX = timeRangeX[2:length(timeRangeX)]
  timeRangeY = timeRangeY[2:length(timeRangeY)]
  timeRangeLabelX = fillUpStr(fillUpDecimals(round(timeRangeX, roundX)))
  timeRangeLabelY = fillUpStr(fillUpDecimals(round(timeRangeY, roundY)))
  
  oneDimKMX = oneDimHistBreakCells(Sdx = Sdx, divPointsX)
  oneDimKMY = oneDimHistBreakCells(Sdx = Sdy, divPointsY)
  oneDimKMY$newVec = oneDimKMY$newVec/sum(oneDimKMY$newVec)
  oneDimKMX$newVec = oneDimKMX$newVec/sum(oneDimKMX$newVec)
  twoDStuff = twoDimHistBreakCells(Sdxdy, divPointsX, divPointsY)
  newMatr = twoDStuff$newMatr; divPX = twoDStuff$divPX; divPY = twoDStuff$divPY
  newMatr[newMatr < 0] = 0
  newMatr = newMatr/sum(newMatr)
  colMatr = grayScale(newMatr, X0 = min(newMatr), X1 = max(newMatr), Y0 = 0.9, Y1 = -20, method = 3,
                      lowerThreshold = 10^(-13), YValueForLowerThreshold = whitestCol)
  colMatr[colMatr < 0] = 0
  
  totalHistMax = max(oneDimKMX$newVec, oneDimKMY$newVec)
  ### note that the scales and gaps are switched because ....
  # gapX = 3 * gridWidthX * scaleGapY
  # gapY = 3 * gridWidthY * scaleGapX
  gapX = 3*min(gridWidthX, gridWidthY) * scaleGapY
  gapY = 3*min(gridWidthX, gridWidthY) * scaleGapX
  startY = 0 - totalHistMax - gapX
  startX = 0 - totalHistMax - gapY
  # startY = 0 - maxY - gapX
  # startX = 0 - maxX - gapY
  
  par(mar = rep(4, 4))
  xlim = c(startX, max(divPointsX))
  ylim = c(startY, max(divPointsY))
  plot(0, 0, xlim = xlim, ylim = ylim, type = "n", main = "", xlab = "", ylab = "", axes = FALSE)
  # plot(0, 0, xlim = , ylim = c(min(startX, startY), totalMax), type = "n", main = "", xlab = "", ylab = "", axes = FALSE)
  widowSize = dev.size("in")
  cexNumbers = (0.7/7)*min(widowSize)*timeLabelScale
  axisLabelCex = (0.9/7)*min(widowSize)*axisLabelScale
  
  # plot1DHistWithRotation(startX = -gapX, startY = 0, rotationRadians = pi/2, newVec = oneDimKMY$newVec, divPX = oneDimKMY$divPX, whitestCol = whitestCol, grayMaxCol = grayMaxCol)
  # plot1DHistWithRotation (startX = 0, startY = -gapY, rotationRadians = 0, oneDimKMX$newVec * (-1), oneDimKMX$divPX, xlab = "X", whitestCol = whitestCol, grayMaxCol = grayMaxCol)
  ### plot(c(-1, 2), c(-1, 2), type = "n")

  plotVector(vectValues = oneDimKMY$newVec, width = oneDimKMY$divPX[,2] - oneDimKMY$divPX[,1],
                         coordX = -gapX, coordY = 0, rotationRadians = pi/2, bordColor = "white")
  plotVector(vectValues = oneDimKMX$newVec * (-1), width = oneDimKMX$divPX[,2] - oneDimKMX$divPX[,1],
                         coordX = 0, coordY = -gapY, rotationRadians = 0, bordColor = "white")
  
  # plot2DHist(startX = 0, startY = 0, newMatr, divPX, divPY, whitestCol = whitestCol, grayMaxCol = grayMaxCol, colMatr = colMatr)
  # plotMatrix(colorMatr = colorMatr, borderCol = "white", coordX = 0, coordY = 0, widthX = divPX[,2] - divPX[,1], widthY = divPY[,2] - divPY[,1])

  colorMatr = matrix(gray(colMatr), nrow = nrow(divPX), ncol = nrow(divPY))
  colorMatr = t(colorMatr)
  colorMatr = colorMatr[nrow(colorMatr):1, ]
  plotMatrix(colorMatr = colorMatr, borderCol = "white", coordX = 0, coordY = 0, widthY = divPY[,2] - divPY[,1], widthX = divPX[,2] - divPX[,1])
  
  # text(x = rep(0, length(timeRangeX)) - gapX/2, y = timeRangeX, label = timeRangeLabelX, srt = 0, cex = cexNumbers)
  # text(y = rep(0, length(timeRangeY)) - gapY/2, x = timeRangeY, label = timeRangeLabelY, srt = 90, cex = cexNumbers)
  text(x = timeRangeX, y = rep(0, length(timeRangeX)) - gapY/2, label = timeRangeLabelX, srt = 90, cex = cexNumbers)
  text(x = rep(0, length(timeRangeY)) - gapX/2, y = timeRangeY, label = timeRangeLabelY, srt = 0, cex = cexNumbers)
  text(x = - gapX/2, y = - gapY/2, label = 0, srt = 135, cex = cexNumbers, font = 1)
  
  mtext(text = XAxisLabel, side = 1, at = mean(timeRangeX[c(1,length(timeRangeX))]), line = -0.5, cex = axisLabelCex) ### -4
  mtext(text = YAxisLabel, side = 2, at = mean(timeRangeY[c(1,length(timeRangeY))]), line = -0.5, cex = axisLabelCex)
  mtext(text = plotLabel, side = 3,  at = mean(timeRangeX[c(1,length(timeRangeX))]), line = 0.5, cex = axisLabelCex, font = 2)
}


###################################################################
###################################################################
#################################   EXAMPLES   ####################
###################################################################
###################################################################
### In the following examples we use data simulated with Frank's copula family with parameter 4.426265, which corresponds to Spearman's correlation of 0.6. For a given restricted region defined by arguments tauX and tauY, function survSpearman() returns: 
###     The user-specified restricted region; 
###     The effective restricted region, which are the values just above the latest observed event times;
###     The highest rank correlation and the restricted correlation.

###################### Load all data
###################### 
folderName = "."

data123 = read.table(file.path(folderName, "data123.csv"))
dataExample = read.table(file.path(folderName, "dataExample.csv"))

###################################################################
#################################   EXAMPLE 1  ####################
###################################################################
### Data are generated such that T_X and T_Y are censored at tau_X=tau_Y=2 (bivariate generalized type-I censoring with the follow-up time 2 for both variables).
###################### 
data1 = data123[1:100,]
data1[1:7, ]

### Because of type-I censoring, the survival probability cannot be fully estimated outside of [0, tau_X) x [0, tau_Y) = [0, 2) x [0, 2), so when computing correlation, it makes sense to choose a restricted region with the same follow-up time, [0, 2) x [0, 2). The function returns the user-specified restricted region as well as the effective restricted region, which are the values just above the latest observed event times,
###################### 
dabrSurf = survDabrowska(data1$X, data1$Y, data1$deltaX, data1$deltaY)$DabrowskaEst
res = survSpearman(bivarSurf = dabrSurf, tauX = 2, tauY = 2)
res[["Restricted region set by user"]]
res[["Effective restricted region"]]

### When computing correlation under generalized type-I censoring, we can either assume the highest rank for points outside of the restricted region and compute $\widehat{\rho}_S^H$ or compute correlation conditional on being in the restricted region, $\widehat{\rho}_{S|\Omega_R}$. Because at least one observation was censored outside of [0, 2) x [0, 2), the probability of being inside the restricted region is less than one and therefore $\widehat{\rho}_S^H$ is not equal to $\widehat{\rho}_{S|\Omega_R}$:
###################### 
res[["Correlation"]]

### One could choose values tau_X and tau_Y that are less than those used to generate the data.  For example, if the analyst choses $\tau_X=\tau_Y = 1.5$, the function returns the following estimates:
###################### 
res =  survSpearman(bivarSurf = dabrSurf, tauX = 1.5, tauY = 1.5)
res

### The user can also choose a restricted region that contains [0, 2) x [0, 2), for example [0, 3) x [0, 3), but this is not advisable because there are no events outside of [0, 2) x [0, 2) so setting a larger restricted region will result in the same correlation values as for [0, 2) x [0, 2),
###################### 
res =  survSpearman(bivarSurf = dabrSurf, tauX = 3, tauY = 3)
res


###################################################################
#################################   EXAMPLE 2  ####################
###################################################################
### In this example, we generate data with random censoring or unrestricted follow-up time.  In this case, $\widehat{\rho}_S^H$ is consistent for the overall Spearman's correlation.  If the largest X and Y correspond to events ($\delta_X=\delta_Y=1$),
###################### 
data2 = data123[101:200,]
data2[data2$X == max(data2$X), c("X", "deltaX")]
data2[data2$Y == max(data2$Y), c("Y", "deltaY")]

### then the survival surface is fully estimated and if the restricted region of interest contains the data support then $\widehat{S}^H(x,y) =\widehat{S}(x,y|\Omega_R) = \widehat{S}(x,y)$ and $\widehat{\rho}_S^H = \widehat{\rho}_{S|\Omega_R}$:
###################### 
dabrSurf = survDabrowska(data2$X, data2$Y, data2$deltaX, data2$deltaY)$DabrowskaEst
res = survSpearman(bivarSurf = dabrSurf, tauX = Inf, tauY = Inf)
res

### In contrast, if the largest $X$ and/or $Y$ correspond to censoring events ($\delta_X$ and/or $\delta_Y=0$), 
###################### 
data2[data2$X == max(data2$X), "deltaX"] = 0

### then the survival surface cannot be estimated after the last censored event and therefore $\widehat{\rho}_S^H$ is not equal to $\widehat{\rho}_{S|\Omega_R}$:
######################
dabrSurf = survDabrowska(data2$X, data2$Y, data2$deltaX, data2$deltaY)$DabrowskaEst
res = survSpearman(bivarSurf = dabrSurf, tauX = Inf, tauY = Inf)
res


###################################################################
#################################   EXAMPLE 3  ####################
###################################################################
### Note that if there is no censoring and $\tau_X=\tau_Y=\infty$, then both estimates are equivalent to Spearman's rank correlation.
######################
data3 = data123[201:300,]
dabrSurf = survDabrowska(X = data3$X, Y = data3$Y, deltaX = data3$deltaX, deltaY = data3$deltaY)$DabrowskaEst
res = survSpearman(bivarSurf = dabrSurf, tauX = Inf, tauY = Inf)
res
cor(data3$X, data3$Y, method = "spearman")

### Finally, even without censoring, the code allows one to compute highest rank and restricted correlations within a user-specified region defined by $\tau_X$ and $\tau_Y$,
######################
res =  survSpearman(bivarSurf = dabrSurf, tauX = qexp(0.75), tauY = qexp(0.75))
res


###################################################################
#################################   EXAMPLE 4  ####################
###################################################################
### In this example we plot bivariate porbability mass function estimated using Dabrowska's method. The data is simulated 500 pairs with Clayton's copula family (parameter 0.5214356) aiming for the overall Spearman's correlation of 0.3. Each variable is distributed exponentially with mean 1. Generalized type I censoring was applied with the median survival time (0.6931472) as a follow-up for both variables.
###################### 
dabrSurf = survDabrowska(X = dataExample$X, Y = dataExample$Y, deltaX = dataExample$deltaX, deltaY = dataExample$deltaY)$DabrowskaEst
res = survSpearman(bivarSurf = dabrSurf, tauX = Inf, tauY = Inf)
res

######################
### Because we have type I censoring, the resulting survival surface is not proper, because it does not diminish all the way to zero. But let's plot its PDF anyway:
gridWidth = 0.05
survPMFPlot(bivarSurf = dabrSurf, gridWidthX = gridWidth, gridWidthY = gridWidth, scaleGapX = 1.5, scaleGapY = 1.5, XAxisLabel = "X", YAxisLabel = "Y", timeLabelScale = 1, axisLabelScale = 1, labelSkipX = 2, labelSkipY = 2, roundX = 2, roundY = 2, plotLabel = "Bivariate PMF")

######################
### Now let's build a proper survival surface by assigning the survival probability to zero after the median survival time:
bivarSurfWithTails = rbind(cbind(dabrSurf, rep(0, nrow(dabrSurf))), rep(0, ncol(dabrSurf)+1))
lastRowVal = as.numeric(rownames(dabrSurf)[nrow(dabrSurf)]) + 2*gridWidth
lastColVal = as.numeric(colnames(dabrSurf)[ncol(dabrSurf)]) + 2*gridWidth
rownames(bivarSurfWithTails) = c(rownames(dabrSurf), as.character(lastRowVal))
colnames(bivarSurfWithTails) = c(colnames(dabrSurf), as.character(lastColVal))
######################
### As a result, we get a PDF with the left-over probability 'tails':
survPMFPlot(bivarSurf = bivarSurfWithTails, gridWidthX = gridWidth, gridWidthY = gridWidth, scaleGapX = 1.5, scaleGapY = 1.5, XAxisLabel = "X", YAxisLabel = "Y", timeLabelScale = 1, axisLabelScale = 1, labelSkipX = 2, labelSkipY = 2, roundX = 2, roundY = 2, plotLabel = "Bivariate PMF")

######################
### Finally, let's plot a conditional PDF withing a restricted region:
condSurf = survRestricted (bivarSurf = dabrSurf, tauX = 0.4, tauY = 0.4)$Sxy
survPMFPlot(bivarSurf = condSurf, gridWidthX = gridWidth, gridWidthY = gridWidth, scaleGapX = 1.5, scaleGapY = 1.5, XAxisLabel = "X", YAxisLabel = "Y", timeLabelScale = 1, axisLabelScale = 1, labelSkipX = 2, labelSkipY = 2, roundX = 2, roundY = 2, plotLabel = "Bivariate PMF")

#### the end




















